==> plugin-mq-client-2.0.8-20200715.100508-7-sources.jar: com/ohaotian/plugin/mq/proxy/ext/aliyunmq/AliyunMqMessageSender.java <==
package com.ohaotian.plugin.mq.proxy.ext.aliyunmq;

import com.aliyun.openservices.ons.api.*;
import com.aliyun.openservices.ons.api.transaction.LocalTransactionExecuter;
import com.aliyun.openservices.ons.api.transaction.TransactionProducer;
import com.aliyun.openservices.ons.api.transaction.TransactionStatus;
import com.ohaotian.plugin.mq.proxy.callback.ProxyLocalTransactionExecuter;
import com.ohaotian.plugin.mq.proxy.callback.ProxySendCallback;
import com.ohaotian.plugin.mq.proxy.ext.ProxyMessageProducerEx;
import com.ohaotian.plugin.mq.proxy.ext.ProxyMqTransactionChecker;
import com.ohaotian.plugin.mq.proxy.internal.ProxyMessageConfig;
import com.ohaotian.plugin.mq.proxy.internal.ProxyMessageException;
import com.ohaotian.plugin.mq.proxy.status.ProxyTransactionStatus;
import com.ohaotian.plugin.mq.proxy.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

public class AliyunMqMessageSender implements ProxyMessageProducerEx, ApplicationContextAware {
    private final Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    private boolean started;
    private Properties mergedProps = new Properties();
    private Producer producer;
    private TransactionProducer transProductor;
    private ProxyMessageConfig messageConfig;
    private final String subject;
    private final Map<ProxyMessageType, Set<String>> typeTagsMapping;
    private AliyunMqTransactionCheckerListener transactionCheckerListener;
    private CacheStore cacheStore;

    public AliyunMqMessageSender(ProxyMessageConfig messageConfig, Map<ProxyMessageType, Set<String>> typeTagsMapping) {
        logger.info("[" + this.getClass().getName() + "] required properties[mq.producerId|mq.accessKey|mq.secretKey|mq.onsAddr|mq.sendMsgTimeoutMillis]");
        this.subject = messageConfig.getSubject();
        this.messageConfig = messageConfig;
        this.typeTagsMapping = typeTagsMapping;
    }

    public ProxySendResult send(ProxyMessage message) {
        checkStatus(message, ProxyMessageType.SYNCHRONIZATION);
        ProxySendResult proxySendResult;
        try {
            SendResult sendResult = producer.send(getMessage(message));
            proxySendResult = getProxySendResult(sendResult);
        } catch (Exception e) {
            throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
        }
        return proxySendResult;
    }

    public ProxySendResult sendInTransaction(final ProxyMessage proxyMessage, final ProxyLocalTransactionExecuter localTransactionExecuter, final Object arg) {
        checkStatus(proxyMessage, ProxyMessageType.TRANSACTION);
        ProxySendResult proxySendResult;
        Message message = getMessage(proxyMessage);
        try {
            SendResult sendResult = transProductor.send(message, new LocalTransactionExecuter() {
                public TransactionStatus execute(Message msg, Object arg) {
                    ProxyTransactionStatus transactionStatus = localTransactionExecuter.exec(proxyMessage, arg);
                    if (transactionStatus == ProxyTransactionStatus.COMMIT) {
                        return TransactionStatus.CommitTransaction;
                    } else if (transactionStatus == ProxyTransactionStatus.ROLLBACK) {
                        return TransactionStatus.RollbackTransaction;
                    } else if (transactionStatus == ProxyTransactionStatus.UNKNOW) {
                        return TransactionStatus.Unknow;
                    }
                    return TransactionStatus.Unknow;
                }
            }, arg);
            proxySendResult = getProxySendResult(sendResult);
            cacheStore.set(message.getMsgID(), ProxyTransactionStatus.COMMIT.toString(), ProxyMqTransactionChecker.MQ_TRAN_CHECK_EXPIRE);
            if (logger.isDebugEnabled()) {
                logger.debug("send tran msg msgId={" + message.getMsgID() + "}  topic=" + proxyMessage.getSubject() + " tag=" + proxyMessage.getTag() + "  body={" + proxyMessage.getContent() + "}  SendResult={" + proxySendResult.getStatus() + "}");
            }
        } catch (Exception e) {
            cacheStore.set(message.getMsgID(), ProxyTransactionStatus.ROLLBACK.toString(), ProxyMqTransactionChecker.MQ_TRAN_CHECK_EXPIRE);
            throw new ProxyMessageException("send subject[" + proxyMessage.getSubject() + "] tag[" + proxyMessage.getTag() + "] error", e);
        }
        return proxySendResult;
    }

    private ProxySendResult getProxySendResult(SendResult sendResult) {
        ProxySendResult proxySendResult;
        proxySendResult = new ProxySendResult();
        proxySendResult.setMsgId(sendResult.getMessageId());
        proxySendResult.setStatus(ProxySendResult.SEND_OK);
        return proxySendResult;
    }

    private Message getMessage(ProxyMessage message) {
        Message sendMessage;
        try {
            sendMessage = new Message(message.getSubject(), message.getTag(), message.getContent().getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Convert rocketmq message error", e);
        }
        return sendMessage;
    }

    public void send(ProxyMessage message, final ProxySendCallback sendCallback) {
        checkStatus(message, ProxyMessageType.ASYNCHRONOUS);
        try {
            producer.sendAsync(getMessage(message), new SendCallback() {
                public void onSuccess(SendResult sendResult) {
                    if (sendCallback != null) {
                        sendCallback.onSuccess(getProxySendResult(sendResult));
                    }
                }

                public void onException(OnExceptionContext context) {
                    if (sendCallback != null) {
                        sendCallback.onException(new ProxyExceptionContext(context.getException()));
                    }
                }
            });
        } catch (Throwable e) {
            if (sendCallback != null) {
                sendCallback.onException(new ProxyExceptionContext(e));
            } else {
                throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
            }
        }
    }

    private void checkStatus(ProxyMessage message, ProxyMessageType proxyMessageType) {
        //todo: checkStatus
        if(true) {
            return;
        }
        if (!this.subject.equals(message.getSubject())) {
            throw new IllegalArgumentException("Unsupported subject[" + message.getSubject() + "].Supported subject[" + this.subject + "].");
        }
        Set<String> allowedTags = typeTagsMapping.get(proxyMessageType);
        if (allowedTags == null || (!allowedTags.contains(message.getTag()) && !allowedTags.contains("*"))) {
            throw new IllegalStateException("Subject[" + message.getSubject() + "]unsupported tag[" + message.getTag() + "].Supported tags " + (allowedTags == null ? "[]" : allowedTags) + " for messageType[" + proxyMessageType + "].");
        }
    }

    public void sendOneway(ProxyMessage message) {
        checkStatus(message, ProxyMessageType.ONEWAY);
        try {
            producer.sendOneway(getMessage(message));
        } catch (Throwable e) {
            throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
        }
    }

    public void startup() {
        Properties config = new Properties();
        Properties consumerProps = messageConfig.getProperties();

        config.put(PropertyKeyConst.ProducerId, ((DefaultProxyMessageConfig) messageConfig).getId());
        config.put(PropertyKeyConst.AccessKey, consumerProps.getProperty("mq.accessKey", mergedProps.getProperty("mq.accessKey")));
        config.put(PropertyKeyConst.SecretKey, consumerProps.getProperty("mq.secretKey", mergedProps.getProperty("mq.secretKey")));
        config.put(PropertyKeyConst.SendMsgTimeoutMillis, consumerProps.getProperty("mq.sendMsgTimeoutMillis", mergedProps.getProperty("mq.sendMsgTimeoutMillis")));
//        config.put(PropertyKeyConst.ONSAddr, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));
//        config.put(PropertyKeyConst.NAMESRV_ADDR, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));

        logger.info("***********是否使用gid***********");
        if(mergedProps.containsKey("mq.supportGID")&&mergedProps.getProperty("mq.supportGID").equals("false")){
            logger.info("***********不使用gid***********");
            config.put(PropertyKeyConst.ProducerId, ((DefaultProxyMessageConfig) messageConfig).getId());
            config.put(PropertyKeyConst.ONSAddr, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));
        }else{
            logger.info("***********使用gid***********"+mergedProps.getProperty("mq.supportGID"));
            config.put(PropertyKeyConst.NAMESRV_ADDR, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));
        }


        if (typeTagsMapping.containsKey(ProxyMessageType.TRANSACTION)) {
            transactionCheckerListener = new AliyunMqTransactionCheckerListener();
            transactionCheckerListener.setCacheStore(cacheStore);
            transProductor = ONSFactory.createTransactionProducer(config, transactionCheckerListener);
        } else {
            transProductor = null;
        }
        boolean createCommonProductor = false;
        if (typeTagsMapping.containsKey(ProxyMessageType.SYNCHRONIZATION)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ASYNCHRONOUS)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ONEWAY)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ORDERED)) {
            createCommonProductor = true;
        }
        if (createCommonProductor) {
            producer = ONSFactory.createProducer(config);
        } else {
            producer = null;
        }

        if (this.producer != null) {
            this.producer.start();
        }
        if (this.transProductor != null) {
            this.transProductor.start();
        }
    }

    public void shutdown() {
        if (this.producer != null) {
            this.producer.shutdown();
        }
        if (this.transProductor != null) {
            this.transProductor.shutdown();
        }
    }

    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        Map<String, Properties> propsMapping = applicationContext.getBeansOfType(Properties.class);
        if (propsMapping != null) {
            for (Properties props : propsMapping.values()) {
                CollectionUtils.mergePropertiesIntoMap(props, mergedProps);
            }
        }
        this.cacheStore = applicationContext.getBean(CacheStore.class);
    }
}

==> plugin-mq-client-2.0.11-20200908.083842-6-sources.jar: com/ohaotian/plugin/mq/proxy/ext/aliyunmq/AliyunMqMessageSender.java <==
package com.ohaotian.plugin.mq.proxy.ext.aliyunmq;

import com.aliyun.openservices.ons.api.*;
import com.aliyun.openservices.ons.api.transaction.LocalTransactionExecuter;
import com.aliyun.openservices.ons.api.transaction.TransactionProducer;
import com.aliyun.openservices.ons.api.transaction.TransactionStatus;
import com.ohaotian.plugin.mq.proxy.callback.ProxyLocalTransactionExecuter;
import com.ohaotian.plugin.mq.proxy.callback.ProxySendCallback;
import com.ohaotian.plugin.mq.proxy.config.ApolloConfigVO;
import com.ohaotian.plugin.mq.proxy.ext.ProxyMessageProducerEx;
import com.ohaotian.plugin.mq.proxy.ext.ProxyMqTransactionChecker;
import com.ohaotian.plugin.mq.proxy.internal.ProxyMessageConfig;
import com.ohaotian.plugin.mq.proxy.internal.ProxyMessageException;
import com.ohaotian.plugin.mq.proxy.status.ProxyTransactionStatus;
import com.ohaotian.plugin.mq.proxy.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.DependsOn;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

public class AliyunMqMessageSender implements ProxyMessageProducerEx, ApplicationContextAware {
    private final Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    private boolean started;
    private Properties mergedProps = new Properties();
    @Autowired
    private static ApolloConfigVO apolloConfig;
    private Producer producer;
    private TransactionProducer transProductor;
    private ProxyMessageConfig messageConfig;
    private final String subject;
    private final Map<ProxyMessageType, Set<String>> typeTagsMapping;
    private AliyunMqTransactionCheckerListener transactionCheckerListener;
    private CacheStore cacheStore;

    public AliyunMqMessageSender(ProxyMessageConfig messageConfig, Map<ProxyMessageType, Set<String>> typeTagsMapping) {
        logger.info("[" + this.getClass().getName() + "] required properties[mq.producerId|mq.accessKey|mq.secretKey|mq.onsAddr|mq.sendMsgTimeoutMillis]");
        this.subject = messageConfig.getSubject();
        this.messageConfig = messageConfig;
        this.typeTagsMapping = typeTagsMapping;
    }

    public ProxySendResult send(ProxyMessage message) {
        checkStatus(message, ProxyMessageType.SYNCHRONIZATION);
        ProxySendResult proxySendResult;
        try {
            SendResult sendResult = producer.send(getMessage(message));
            proxySendResult = getProxySendResult(sendResult);
        } catch (Exception e) {
            throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
        }
        return proxySendResult;
    }

    public ProxySendResult sendInTransaction(final ProxyMessage proxyMessage, final ProxyLocalTransactionExecuter localTransactionExecuter, final Object arg) {
        checkStatus(proxyMessage, ProxyMessageType.TRANSACTION);
        ProxySendResult proxySendResult;
        Message message = getMessage(proxyMessage);
        try {
            SendResult sendResult = transProductor.send(message, new LocalTransactionExecuter() {
                public TransactionStatus execute(Message msg, Object arg) {
                    ProxyTransactionStatus transactionStatus = localTransactionExecuter.exec(proxyMessage, arg);
                    if (transactionStatus == ProxyTransactionStatus.COMMIT) {
                        return TransactionStatus.CommitTransaction;
                    } else if (transactionStatus == ProxyTransactionStatus.ROLLBACK) {
                        return TransactionStatus.RollbackTransaction;
                    } else if (transactionStatus == ProxyTransactionStatus.UNKNOW) {
                        return TransactionStatus.Unknow;
                    }
                    return TransactionStatus.Unknow;
                }
            }, arg);
            proxySendResult = getProxySendResult(sendResult);
            cacheStore.set(message.getMsgID(), ProxyTransactionStatus.COMMIT.toString(), ProxyMqTransactionChecker.MQ_TRAN_CHECK_EXPIRE);
            if (logger.isDebugEnabled()) {
                logger.debug("send tran msg msgId={" + message.getMsgID() + "}  topic=" + proxyMessage.getSubject() + " tag=" + proxyMessage.getTag() + "  body={" + proxyMessage.getContent() + "}  SendResult={" + proxySendResult.getStatus() + "}");
            }
        } catch (Exception e) {
            cacheStore.set(message.getMsgID(), ProxyTransactionStatus.ROLLBACK.toString(), ProxyMqTransactionChecker.MQ_TRAN_CHECK_EXPIRE);
            throw new ProxyMessageException("send subject[" + proxyMessage.getSubject() + "] tag[" + proxyMessage.getTag() + "] error", e);
        }
        return proxySendResult;
    }

    private ProxySendResult getProxySendResult(SendResult sendResult) {
        ProxySendResult proxySendResult;
        proxySendResult = new ProxySendResult();
        proxySendResult.setMsgId(sendResult.getMessageId());
        proxySendResult.setStatus(ProxySendResult.SEND_OK);
        return proxySendResult;
    }

    private Message getMessage(ProxyMessage message) {
        Message sendMessage;
        try {
            sendMessage = new Message(message.getSubject(), message.getTag(), message.getContent().getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Convert rocketmq message error", e);
        }
        return sendMessage;
    }

    public void send(ProxyMessage message, final ProxySendCallback sendCallback) {
        checkStatus(message, ProxyMessageType.ASYNCHRONOUS);
        try {
            producer.sendAsync(getMessage(message), new SendCallback() {
                public void onSuccess(SendResult sendResult) {
                    if (sendCallback != null) {
                        sendCallback.onSuccess(getProxySendResult(sendResult));
                    }
                }

                public void onException(OnExceptionContext context) {
                    if (sendCallback != null) {
                        sendCallback.onException(new ProxyExceptionContext(context.getException()));
                    }
                }
            });
        } catch (Throwable e) {
            if (sendCallback != null) {
                sendCallback.onException(new ProxyExceptionContext(e));
            } else {
                throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
            }
        }
    }

    private void checkStatus(ProxyMessage message, ProxyMessageType proxyMessageType) {
        //todo: checkStatus
        if(true) {
            return;
        }
        if (!this.subject.equals(message.getSubject())) {
            throw new IllegalArgumentException("Unsupported subject[" + message.getSubject() + "].Supported subject[" + this.subject + "].");
        }
        Set<String> allowedTags = typeTagsMapping.get(proxyMessageType);
        if (allowedTags == null || (!allowedTags.contains(message.getTag()) && !allowedTags.contains("*"))) {
            throw new IllegalStateException("Subject[" + message.getSubject() + "]unsupported tag[" + message.getTag() + "].Supported tags " + (allowedTags == null ? "[]" : allowedTags) + " for messageType[" + proxyMessageType + "].");
        }
    }

    public void sendOneway(ProxyMessage message) {
        checkStatus(message, ProxyMessageType.ONEWAY);
        try {
            producer.sendOneway(getMessage(message));
        } catch (Throwable e) {
            throw new ProxyMessageException("send subject[" + message.getSubject() + "] tag[" + message.getTag() + "] error", e);
        }
    }
    @DependsOn("apolloConfigVO")
    public void startup() {
        Properties config = new Properties();
        Properties consumerProps = messageConfig.getProperties();
        config.put(PropertyKeyConst.ProducerId, ((DefaultProxyMessageConfig) messageConfig).getId());
        config.put(PropertyKeyConst.AccessKey, consumerProps.getProperty("mq.accessKey", apolloConfig.getAccessKey()));
        config.put(PropertyKeyConst.SecretKey, consumerProps.getProperty("mq.secretKey", apolloConfig.getSecretKey()));
        config.put(PropertyKeyConst.SendMsgTimeoutMillis, consumerProps.getProperty("mq.sendMsgTimeoutMillis", apolloConfig.getSendMsgTimeoutMillis()));
//        config.put(PropertyKeyConst.ONSAddr, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));
//        config.put(PropertyKeyConst.NAMESRV_ADDR, consumerProps.getProperty("mq.onsAddr", mergedProps.getProperty("mq.onsAddr")));

        logger.info("***********是否使用gid***********");
        if(StringUtils.isEmpty(apolloConfig.getSupportGID())){
            logger.info("***********不使用gid***********");
            config.put(PropertyKeyConst.ProducerId, ((DefaultProxyMessageConfig) messageConfig).getId());
            config.put(PropertyKeyConst.ONSAddr, consumerProps.getProperty("mq.onsAddr", apolloConfig.getOnsAddr()));
        }else{
            logger.info("***********使用gid***********"+apolloConfig.getSupportGID());
            config.put(PropertyKeyConst.NAMESRV_ADDR, consumerProps.getProperty("mq.onsAddr", apolloConfig.getOnsAddr()));
        }


        if (typeTagsMapping.containsKey(ProxyMessageType.TRANSACTION)) {
            transactionCheckerListener = new AliyunMqTransactionCheckerListener();
            transactionCheckerListener.setCacheStore(cacheStore);
            transProductor = ONSFactory.createTransactionProducer(config, transactionCheckerListener);
        } else {
            transProductor = null;
        }
        boolean createCommonProductor = false;
        if (typeTagsMapping.containsKey(ProxyMessageType.SYNCHRONIZATION)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ASYNCHRONOUS)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ONEWAY)) {
            createCommonProductor = true;
        }
        if (!createCommonProductor && typeTagsMapping.containsKey(ProxyMessageType.ORDERED)) {
            createCommonProductor = true;
        }
        if (createCommonProductor) {
            producer = ONSFactory.createProducer(config);
        } else {
            producer = null;
        }

        if (this.producer != null) {
            this.producer.start();
        }
        if (this.transProductor != null) {
            this.transProductor.start();
        }
    }

    public void shutdown() {
        if (this.producer != null) {
            this.producer.shutdown();
        }
        if (this.transProductor != null) {
            this.transProductor.shutdown();
        }
    }

    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        Map<String, Properties> propsMapping = applicationContext.getBeansOfType(Properties.class);
        if (propsMapping != null) {
            for (Properties props : propsMapping.values()) {
                CollectionUtils.mergePropertiesIntoMap(props, mergedProps);
            }
        }
        this.cacheStore = applicationContext.getBean(CacheStore.class);
    }
}
